@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 --------------------------------------------------------------------------------
 {- |
 Module      :  Numeric.LinearAlgebra.HMatrix
@@ -19,6 +20,9 @@ module Numeric.LinearAlgebra.HMatrix (
 import Numeric.LinearAlgebra
 import Internal.Util
 import Internal.Algorithms(cholSH, mbCholSH, eigSH', eigenvaluesSH', geigSH')
+#if MIN_VERSION_base(4,11,0)
+import Prelude hiding ((<>))
+#endif
 
 infixr 8 <·>
 (<·>) :: Numeric t => Vector t -> Vector t -> t
