@@ -32,8 +32,11 @@ import Internal.Element
 import Internal.Numeric
 import qualified Data.Monoid as M
 import Data.List(partition)
+import qualified Data.Foldable as F
+import qualified Data.Semigroup as S
 import Internal.Chain
 
+
 -------------------------------------------------------------------
 
 instance Container Matrix a => Eq (Matrix a) where
@@ -84,6 +87,11 @@ adaptScalarM f1 f2 f3 x y
     | isScalar y = f3 x (y @@>(0,0) )
     | otherwise = f2 x y
 
+instance (Container Vector t, Eq t, Num (Vector t), Product t) => S.Semigroup (Matrix t)
+  where
+    (<>) = mappend
+    sconcat = mconcat . F.toList
+
 instance (Container Vector t, Eq t, Num (Vector t), Product t) => M.Monoid (Matrix t)
   where
     mempty = 1
